@@ -130,7 +130,7 @@ class Gui(abstract.LessonbasedGui):
         hbox.pack_start(table, False)
         hsep = gtk.HSeparator()
         table.attach(hsep, 0, 5, 1, 2, xoptions=gtk.FILL,
-                     yoptions=0, ypadding=gnome.ui.PAD_SMALL)
+                     yoptions=gtk.FILL, xpadding=0, ypadding=gnome.ui.PAD_SMALL)
         self.g_vsep1 = gtk.VSeparator()
         table.attach(self.g_vsep1, 1, 2, 0, 4,
                      xoptions=0, xpadding=gnome.ui.PAD)
