@@ -41,13 +41,12 @@ assignmentlist peek: 'NAME' '='
 # P singchord-1 sparer jeg ikke noe p  ha en peek2_type(t1, t2)
 # som tester de to neste token.
 
+if __name__ == "__main__":
+    import testsetup
+
 import sys
 import i18n
 
-if __name__ == "__main__" or __name__ == "dataparser":
-    sys.path.append("src")
-    i18n.setup_srcdir()
-
 import os, os.path
 import re
 
@@ -136,6 +135,9 @@ class Question(dict):
         return "[no name]"
     def get_music(self, varname='music'):
         return self['m_lessonfile'].prepare_music_from_string(self, varname)
+    def is_transposable(question):
+        return question['music'].m_musictype in ('chord', 'satb', 'voice', 'rvoice')
+
 
 class DataparserException(Exception):
     def __init__(self, filename, lexer):
