@@ -233,6 +233,11 @@ class Gui(abstract.IntervalGui):
         return self.get_list('intervals')
     def click_on_interval(self, mouse_button, interval, midi_int):
         if mouse_button == 1:
+            # Do nothing if the user clicks on the anwer buttons or on
+            # another input widget before the test has started.
+            if self.m_t.m_app.m_test_mode and self.m_t.q_status == const.QSTATUS_NO:
+                self.g_flashbar.flash(_("Click 'Start test' to begin."))
+                return
             # we have to ignore intervals greater than decime, because
             # the statistics viewer can't handle it. It is no big deal
             # to fix it, but then the statistics can become almost unreadable
