@@ -23,6 +23,7 @@ import gettext
 import locale
 import os
 import sys
+import cfg
 
 def _i(s):
     """
@@ -59,12 +60,46 @@ def langs():
     return retval
 
 def setup(prefix):
-    if sys.platform == 'win32':
-        if 'LANGUAGE' not in os.environ:
+    # Gettext and gtk+ chech the variables in this order.
+    varlist = ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG')
+    for varname in varlist:
+        if varname in os.environ:
+            print "Found environment var:", varname, os.environ[varname]
+        else:
+            print "Environment var not found:", varname
+    config_locale = cfg.get_string("app/lc_messages")
+    if not config_locale:
+        config_locale = 'system default'
+    if config_locale != 'system default':
+        print "Set language from cfg module:", cfg.get_string("app/lc_messages")
+        for n in varlist:
+            os.environ[n] = config_locale
+    if (sys.platform == 'win32') and (config_locale == 'system default'):
+        print "Trying system default"
+        envar = None
+        for varname in varlist:
+            if varname in os.environ:
+                envar = varname
+                break
+        print "Env var found:", envar
+        if not envar:
+            # We have to set the value ourselves if we don't have
+            # a environment variable set.
+            print "defaultlocale:", locale.getdefaultlocale()
             s = locale.getdefaultlocale()[0]
-            if '_' in s:
-                s = s.split("_")[1]
-            os.environ['LANGUAGE'] = s
+            print "Trying:", s, locale.normalize(s)
+            if s == 'nb_NO':
+                s = 'no_NO'
+            if s:
+                s = locale.normalize(s)
+                os.environ['LANGUAGE'] = s
+		print "Satte LANGUAGE:", os.environ['LANGUAGE']
+    for varname in varlist:
+        if varname in os.environ:
+            print "Found environment var:", varname, os.environ[varname]
+        else:
+            print "Environment var not found:", varname
+
     gettext.install('solfege', os.path.join(prefix, 'share', 'locale'), unicode=1)
     __builtins__['_i'] = _i
 
