@@ -17,18 +17,25 @@
 
 from __future__ import division
 
-import gtk, gnome.ui, gnome.canvas
+import gtk, gnome.ui
+try:
+    import gnomecanvas
+    canvasmodule = gnomecanvas
+except ImportError:
+    import gnome.canvas
+    canvasmodule = gnome.canvas
+
 import mpd
 import soundcard
 import cfg
-import configureoutput
+import runtime
 
-class PianoKeyboard(gnome.canvas.Canvas):
+class PianoKeyboard(canvasmodule.Canvas):
     def __init__(self, lowest_tone, highest_tone, size):
         """
         The lowest and highest tones should be without accidentals.
         """
-        gnome.canvas.Canvas.__init__(self)
+        canvasmodule.Canvas.__init__(self)
         self.m_lowest_tone = lowest_tone
         self.m_highest_tone = highest_tone
         lowp = mpd.MusicalPitch.new_from_notename(lowest_tone)
@@ -49,19 +56,19 @@ class PianoKeyboard(gnome.canvas.Canvas):
                         self.m_blacks.append(p.semitone_pitch()+1)
                     else:
                         self.m_blacks.append(None)
-        self.m_keyboard = self.root().add(gnome.canvas.CanvasGroup)
+        self.m_keyboard = self.root().add(canvasmodule.CanvasGroup)
         self.m_w, self.m_h, self.m_h2 = size
         for x in range(len(self.m_whites)):
-            g = self.m_keyboard.add(gnome.canvas.CanvasGroup)
-            g.add(gnome.canvas.CanvasRect,
+            g = self.m_keyboard.add(canvasmodule.CanvasGroup)
+            g.add(canvasmodule.CanvasRect,
                   x1=x*self.m_w, y1=0,
                   x2=(x+1)*self.m_w, y2=self.m_h,
                   fill_color='white', outline_color='black')
             g.connect('event', self.on_event, self.m_whites[x])
         for x in range(len(self.m_blacks)):
             if self.m_blacks[x]:
-                g = self.m_keyboard.add(gnome.canvas.CanvasGroup)
-                g.add(gnome.canvas.CanvasRect,
+                g = self.m_keyboard.add(canvasmodule.CanvasGroup)
+                g.add(canvasmodule.CanvasRect,
                       x1=(x+0.65)*self.m_w, y1=0,
                       x2=(x+1.35)*self.m_w, y2=self.m_h2,
                       fill_color='black', outline_color='black')
@@ -79,8 +86,8 @@ class PianoOctaveWithAccelName(PianoKeyboard):
         PianoKeyboard.__init__(self, 'c', 'b', (30, 100, 60))
         self.m_callback = callback
         for x in range(7):
-            g = self.m_keyboard.add(gnome.canvas.CanvasGroup)
-            g.add(gnome.canvas.CanvasText,
+            g = self.m_keyboard.add(canvasmodule.CanvasGroup)
+            g.add(canvasmodule.CanvasText,
                   text=keys[[0, 2, 4, 5, 7, 9, 11][x]],
                   x=(x+0.5)*self.m_w,
                   y=self.m_h-20,
@@ -89,8 +96,8 @@ class PianoOctaveWithAccelName(PianoKeyboard):
             g.connect('event', self.on_event,
                    mpd.notename_to_int(['c', 'd', 'e', 'f', 'g', 'a', 'b'][x]))
         for x in range(5):
-            g = self.m_keyboard.add(gnome.canvas.CanvasGroup)
-            g.add(gnome.canvas.CanvasText, text=keys[[1, 3, 6, 8, 10][x]],
+            g = self.m_keyboard.add(canvasmodule.CanvasGroup)
+            g.add(canvasmodule.CanvasText, text=keys[[1, 3, 6, 8, 10][x]],
                   x=[1, 2, 4, 5, 6][x]*self.m_w,
                   y=self.m_h2-20,
                   font="Sans 24", fill_color='white')
@@ -138,18 +145,18 @@ class IntervalPianoWidget(PianoKeyboard):
     def mark(self, midi_int):
         x, y = self.midi_int_to_key(midi_int)
         if (x, y) not in self.m_marks:
-            g = self.m_keyboard.add(gnome.canvas.CanvasGroup)
+            g = self.m_keyboard.add(canvasmodule.CanvasGroup)
             g.connect('event', self.on_event, midi_int)
             self.m_marks[x, y] = g
             if y == 1:
                 h1 = (self.m_h - self.m_h2)*0.25 + self.m_h2
                 h2 = (self.m_h - self.m_h2)*0.75 + self.m_h2
-                g.add(gnome.canvas.CanvasEllipse,
+                g.add(canvasmodule.CanvasEllipse,
                       x1=x*self.m_w+3, x2=(x+1)*self.m_w-3,
                       y1=h1, y2=h2,
                       outline_color='black', fill_color='red')
             elif y == 0:
-                g.add(gnome.canvas.CanvasEllipse,
+                g.add(canvasmodule.CanvasEllipse,
                       x1=(x+0.75)*self.m_w, x2=(x+1.25)*self.m_w,
                       y1=10, y2=20,
                       outline_color='white', fill_color='red')
@@ -239,13 +246,13 @@ class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
         else:
             self.m_buttons[1].grab_focus()
 
-class AbstractGuitarWidget(gnome.canvas.Canvas):
+class AbstractGuitarWidget(canvasmodule.Canvas):
     def __init__(self, callback, strings,
                             string_thickness=(1, 1, 1, 1, 1, 1)):
         assert len(strings) == len(string_thickness)
-        gnome.canvas.Canvas.__init__(self, 1)
+        canvasmodule.Canvas.__init__(self, 1)
         self.m_callback = callback
-        self.m_guitar = self.root().add(gnome.canvas.CanvasGroup)
+        self.m_guitar = self.root().add(canvasmodule.CanvasGroup)
         self.m_guitar.connect('event', self.event_callback)
         self.m_clicked_tones = []
         self.m_shadow = None
@@ -276,51 +283,51 @@ class AbstractGuitarWidget(gnome.canvas.Canvas):
         self.m_neckw = self.m_neckborder \
                        + (self.m_numstring-1)*self.m_stringdist \
                        + 1 + self.m_neckborder
-        self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+        self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                           y1=0, y2=self.m_neckw,
                       fill_color='black')
         px = self.m_fretdist[0]
-        self.m_guitar.add(gnome.canvas.CanvasRect, x1=px, x2=px+1,
+        self.m_guitar.add(canvasmodule.CanvasRect, x1=px, x2=px+1,
                           y1=0, y2=self.m_neckw, fill_color='white')
-        self.m_guitar.add(gnome.canvas.CanvasRect, x1=px+1, x2=px+5,
+        self.m_guitar.add(canvasmodule.CanvasRect, x1=px+1, x2=px+5,
                           y1=0, y2=self.m_neckw, fill_color='gray')
-        self.m_guitar.add(gnome.canvas.CanvasRect, x1=px+5, x2=px+6,
+        self.m_guitar.add(canvasmodule.CanvasRect, x1=px+5, x2=px+6,
                           y1=0, y2=self.m_neckw, fill_color='dark gray')
         for w in self.m_fretdist[1:]:
             px = px + w
-            self.m_guitar.add(gnome.canvas.CanvasRect, x1=px, x2=px+1, y1=0, y2=self.m_neckw,
+            self.m_guitar.add(canvasmodule.CanvasRect, x1=px, x2=px+1, y1=0, y2=self.m_neckw,
                               fill_color='white')
-            self.m_guitar.add(gnome.canvas.CanvasRect, x1=px+1, x2=px+2,
+            self.m_guitar.add(canvasmodule.CanvasRect, x1=px+1, x2=px+2,
                               y1=0, y2=self.m_neckw, fill_color='gray')
-            self.m_guitar.add(gnome.canvas.CanvasRect, x1=px+2, x2=px+3,
+            self.m_guitar.add(canvasmodule.CanvasRect, x1=px+2, x2=px+3,
                               y1=0, y2=self.m_neckw, fill_color='dark gray')
         for y in range(self.m_numstring):
             # the strings
             if string_thickness[y] == 1:
-                self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+                self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                   y1=self.m_neckborder+y*self.m_stringdist,
                   y2=self.m_neckborder+y*self.m_stringdist+1,
                   fill_color='gray')
             elif string_thickness[y] == 2:
-                self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+                self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                   y1=self.m_neckborder+y*self.m_stringdist,
                   y2=self.m_neckborder+y*self.m_stringdist+1,
                   fill_color='white')
-                self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+                self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                   y1=self.m_neckborder+y*self.m_stringdist+1,
                   y2=self.m_neckborder+y*self.m_stringdist+2,
                   fill_color='dark gray')
             else:
-                self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+                self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                   y1=self.m_neckborder+y*self.m_stringdist-1,
                   y2=self.m_neckborder+y*self.m_stringdist,
                   fill_color='white')
 
-                self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+                self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                   y1=self.m_neckborder+y*self.m_stringdist,
                   y2=self.m_neckborder+y*self.m_stringdist+1,
                   fill_color='gray')
-                self.m_guitar.add(gnome.canvas.CanvasRect, x1=0, x2=self.m_neckl,
+                self.m_guitar.add(canvasmodule.CanvasRect, x1=0, x2=self.m_neckl,
                   y1=self.m_neckborder+y*self.m_stringdist+1,
                   y2=self.m_neckborder+y*self.m_stringdist+2,
                   fill_color='dark gray')
@@ -334,8 +341,8 @@ class AbstractGuitarWidget(gnome.canvas.Canvas):
         #FIXME this could be shared with the other instrument widgets.
         self.m_clicked_tones.pop()
     def white_dot(self, x, y):
-        dot = self.m_guitar.add(gnome.canvas.CanvasGroup)
-        dot.add(gnome.canvas.CanvasEllipse, x1=-5, x2=5, y1=-4, y2=6, fill_color='white')
+        dot = self.m_guitar.add(canvasmodule.CanvasGroup)
+        dot.add(canvasmodule.CanvasEllipse, x1=-5, x2=5, y1=-4, y2=6, fill_color='white')
         dot.move(self.m_xlist[x] + self.m_fretdist[x+1] / 2 + 1,
                  self.m_ylist[y])
     def event_callback(self, group, event):
@@ -369,9 +376,9 @@ class AbstractGuitarWidget(gnome.canvas.Canvas):
     def shadow(self, x, y):
         if self.m_shadow:
             self.m_shadow.destroy()
-        self.m_shadow = self.m_guitar.add(gnome.canvas.CanvasGroup)
+        self.m_shadow = self.m_guitar.add(canvasmodule.CanvasGroup)
         self.m_shadow.set_data('name', 'red_dot')
-        self.m_shadow.add(gnome.canvas.CanvasEllipse, x1=-5, x2=5,
+        self.m_shadow.add(canvasmodule.CanvasEllipse, x1=-5, x2=5,
                       y1=-5, y2=5, outline_color='black', fill_color='red')
         self.m_shadow.move(self.m_xlist[x]-self.m_fretdist[x]/2,
                          self.m_ylist[y]-self.m_stringdist/2)
@@ -388,9 +395,9 @@ class AbstractGuitarWidget(gnome.canvas.Canvas):
                 self._set_mark(note - self.m_stringtuning[idx], idx)
     def _set_mark(self, x, y):
         assert (x, y) not in self.m_mark_dict
-        mark = self.m_guitar.add(gnome.canvas.CanvasGroup)
+        mark = self.m_guitar.add(canvasmodule.CanvasGroup)
         self.m_mark_dict[(x, y)] = mark
-        mark.add(gnome.canvas.CanvasEllipse, x1=-7, x2=7,
+        mark.add(canvasmodule.CanvasEllipse, x1=-7, x2=7,
                  y1=-7, y2=7, outline_color='black', fill_color='green')
         mark.move(self.m_xlist[x]-self.m_fretdist[x]/2,
                   self.m_ylist[y]-self.m_stringdist/2)
@@ -410,9 +417,9 @@ class GuitarWidget(AbstractGuitarWidget):
             self.white_dot(x, y)
 
 
-class AccordionWidget(gnome.canvas.Canvas):
+class AccordionWidget(canvasmodule.Canvas):
     def __init__(self, callback, keyboard_system):
-        gnome.canvas.Canvas.__init__(self, 1)
+        canvasmodule.Canvas.__init__(self, 1)
         self.m_callback = callback
         self.m_clicked_tones = []
         self.m_mark_dict = {}
@@ -476,7 +483,7 @@ class AccordionWidget(gnome.canvas.Canvas):
             if mpd.compare_notenames(self.m_highest_tone, v[-1]) < 0:
                 self.m_highest_tone = v[-1]
 
-        self.m_keyboard = kb = self.root().add(gnome.canvas.CanvasGroup)
+        self.m_keyboard = kb = self.root().add(canvasmodule.CanvasGroup)
         self.m_button_radius = 9
         self.m_button_xdist = 20
         self.m_button_ydist = 18
@@ -485,7 +492,7 @@ class AccordionWidget(gnome.canvas.Canvas):
         bb = len(self.m_notenames)*self.m_button_ydist
         tb = -self.m_button_ydist
         bw = 7
-        kb.add(gnome.canvas.CanvasPolygon, points=(lb-bw, -self.m_button_ydist,
+        kb.add(canvasmodule.CanvasPolygon, points=(lb-bw, -self.m_button_ydist,
                                   lb-bw, tb,
                                   lb-bw, bb-bw,
                                   lb, bb,
@@ -496,7 +503,7 @@ class AccordionWidget(gnome.canvas.Canvas):
         for row in range(len(self.m_notenames)):
             sx = (not row % 2) * self.m_button_xdist / 2
             for notename in self.m_notenames[row]:
-                btn = kb.add(gnome.canvas.CanvasGroup)
+                btn = kb.add(canvasmodule.CanvasGroup)
                 btn.set_data('midi_int', mpd.notename_to_int(notename))
                 btn.connect('event', self.event_callback, mpd.notename_to_int(notename))
                 if len(notename) >= 3 and notename[1:][:2] in ["is", "es"]:
@@ -505,15 +512,15 @@ class AccordionWidget(gnome.canvas.Canvas):
                 else:
                     color = "white"
                     border = "black"
-                btn.add(gnome.canvas.CanvasEllipse, x1=-self.m_button_radius,
+                btn.add(canvasmodule.CanvasEllipse, x1=-self.m_button_radius,
                         x2=self.m_button_radius, y1=-self.m_button_radius,
                         y2=self.m_button_radius, fill_color=color,
                         outline_color=border)
                 if (notename[1:][:2] not in ("is", "es")) \
                    and (notename[0] in ("c", "f")):
-                    btn.add(gnome.canvas.CanvasLine, points=(-self.m_button_radius/2, -2,
+                    btn.add(canvasmodule.CanvasLine, points=(-self.m_button_radius/2, -2,
                          self.m_button_radius/2+1, -2), fill_color="gray")
-                    btn.add(gnome.canvas.CanvasLine, points=(-self.m_button_radius/2, 2,
+                    btn.add(canvasmodule.CanvasLine, points=(-self.m_button_radius/2, 2,
                          self.m_button_radius/2+1, 2), fill_color="gray")
                 btn.move(sx, row*self.m_button_ydist)
                 sx = sx + self.m_button_xdist
@@ -541,7 +548,7 @@ class AccordionWidget(gnome.canvas.Canvas):
         for rownum in range(len(self.m_notenames)):
             for n in range(len(self.m_notenames[rownum])):
                 if mpd.notename_to_int(self.m_notenames[rownum][n]) == midi_int:
-                    g = self.m_keyboard.add(gnome.canvas.CanvasEllipse, x1=-5, x2=5,
+                    g = self.m_keyboard.add(canvasmodule.CanvasEllipse, x1=-5, x2=5,
                       y1=-5, y2=5, outline_color='black', fill_color='red')
                     g.connect('event', self.event_callback,
                               mpd.notename_to_int(self.m_notenames[rownum][n]))
@@ -557,7 +564,7 @@ class AccordionWidget(gnome.canvas.Canvas):
     def know_directions(self):
         return 1
 
-if configureoutput.WITH_GNOME:
+if runtime.has_gnome_canvas():
     inputwidget_names = [_("Buttons"), _("Piano"), _("Guitar"), _("Bass"),
                  _("5 string bass"), _("6 string bass"),
                  _("Accordion B griff"), _("Accordion C griff"),
@@ -566,27 +573,27 @@ else:
     inputwidget_names = [_("Buttons")]
 
 def name_to_inputwidget(name, callback):
-        if name == _("Piano"):
-            w = IntervalPianoWidget(callback)
-        elif name == _("Accordion B griff"):
-            w = AccordionWidget(callback, 'norwegian')
-        elif name == _("Accordion C griff"):
-            w = AccordionWidget(callback, 'swedish')
-        elif name == _("Accordion (system used in Finland)"):
-            w = AccordionWidget(callback, 'finnish')
-        elif name == _("Guitar"):
-            w = GuitarWidget(callback, ["e'", "b", "g", "d", "a,", "e,"],
-                                       (1, 1, 1, 2, 2, 3))
-        elif name == _("Bass"):
-            w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,"],
-                                     (1, 1, 2, 3))
-        elif name == _("5 string bass"):
-            w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,", "b,,,"],
-                                     (1, 1, 2, 3, 3))
-        elif name == _("6 string bass"):
-            w = AbstractGuitarWidget(callback, ["c", "g,", "d,", "a,,", "e,,", "b,,,"],
-                                     (1, 1, 2, 2, 3, 3))
-        else:#if name == _("Buttons"):
-            w = IntervalButtonsWidget(callback)
-        return w
+    if name == _("Piano"):
+        w = IntervalPianoWidget(callback)
+    elif name == _("Accordion B griff"):
+        w = AccordionWidget(callback, 'norwegian')
+    elif name == _("Accordion C griff"):
+        w = AccordionWidget(callback, 'swedish')
+    elif name == _("Accordion (system used in Finland)"):
+        w = AccordionWidget(callback, 'finnish')
+    elif name == _("Guitar"):
+        w = GuitarWidget(callback, ["e'", "b", "g", "d", "a,", "e,"],
+                                   (1, 1, 1, 2, 2, 3))
+    elif name == _("Bass"):
+        w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,"],
+                                 (1, 1, 2, 3))
+    elif name == _("5 string bass"):
+        w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,", "b,,,"],
+                                 (1, 1, 2, 3, 3))
+    elif name == _("6 string bass"):
+        w = AbstractGuitarWidget(callback, ["c", "g,", "d,", "a,,", "e,,", "b,,,"],
+                                 (1, 1, 2, 2, 3, 3))
+    else:#if name == _("Buttons"):
+        w = IntervalButtonsWidget(callback)
+    return w
 
