@@ -50,6 +50,9 @@ opt_parser.add_option('--without-gtkhtml', action='store_true',
 opt_parser.add_option('--without-gnome', action='store_true', 
     dest='without_gnome',
     help='Run without the gnome module even if it is installed')
+opt_parser.add_option('--show-gtk-warnings', action='store_true',
+    dest='show_gtk_warnings',
+    help='Show GtkWarnings in the traceback window')
 
 
 options, args = opt_parser.parse_args()
@@ -143,7 +146,7 @@ import tracebackwindow
 # redirect error messages to a window that will popup if
 # something bad happens.
 
-sys.stderr = tracebackwindow.TracebackWindow()
+sys.stderr = tracebackwindow.TracebackWindow(options.show_gtk_warnings)
 
 import const
 import app
@@ -465,9 +468,11 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         s += "</menubar>"
         self.m_practise_and_test_merge_id = self.g_ui_manager.add_ui_from_string(s)
     def show_help_on_current(self):
-        #assert self.m_help_on_current_merge_id == None
+        """
+        Show the "Help on current exercise F1" menu entry on the
+        Help menu.
+        """
         if self.m_help_on_current_merge_id:
-            # Already visible
             return
         self.m_help_on_current_merge_id = self.g_ui_manager.add_ui_from_string("""
 <menubar name='Menubar'>
@@ -476,6 +481,10 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
   </menu>
 </menubar>""")
     def hide_help_on_current(self):
+        """
+        Hide the "Help on current exercise F1" menu entry on
+        the Help menu.
+        """
         if not self.m_help_on_current_merge_id:
             return
         self.g_ui_manager.remove_ui(self.m_help_on_current_merge_id)
